extern crate clap;

use clap::{ArgGroup, Parser};

mod codegen;
mod dynconst;
mod env;
mod intrinsics;
mod labeled_builder;
mod parser;
mod semant;
mod ssa;
mod types;

use codegen::*;
use juno_scheduler::schedule;

extern crate hercules_ir;

#[derive(Parser)]
#[clap(author, version, about, long_about = None)]
#[clap(group(
        ArgGroup::new("scheduling")
            .required(false)
            .args(&["schedule", "default_schedule", "no_schedule"])))]
struct Cli {
    src_file : String,
    #[clap(short, long, value_name = "FILE")]
    schedule : Option<String>,
    #[clap(short, long)]
    default_schedule : bool,
    #[clap(short, long)]
    no_schedule : bool,
}

fn main() {
    let args = Cli::parse();
    let prg = semant::parse_and_analyze(args.src_file);
    match prg {
        Ok(prg) => {
            let (module, func_info) = codegen_program(prg);

            let mut pm =
                if args.default_schedule {
                    // TODO: Default Schedule Options
                    let mut pm = hercules_opt::pass::PassManager::new(module);
                    pm.make_plans();
                    pm
                } else {
                    match args.schedule {
                        None => {
                            hercules_opt::pass::PassManager::new(module)
                        },
                        Some(file) => {
                            // TODO: Use this schedule
                            match schedule(&module, func_info, file) {
                                Ok(plans) => { 
                                    let mut pm = hercules_opt::pass::PassManager::new(module);
                                    pm.set_plans(plans);
                                    pm
                                },
                                Err(msg) => {
                                    eprintln!("Errors during scheduling\n{}", msg);
                                    return;
                                },
                            }
                        },
                    }
                };

            pm.add_pass(hercules_opt::pass::Pass::Verify);
            pm.add_pass(hercules_opt::pass::Pass::PhiElim);
            pm.add_pass(hercules_opt::pass::Pass::Verify);
            pm.add_pass(hercules_opt::pass::Pass::CCP);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::GVN);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::Xdot(true));
            pm.add_pass(hercules_opt::pass::Pass::Forkify);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::Xdot(true));
            let _ = pm.run_passes();
        },
        Err(errs) => {
            for err in errs{
                eprintln!("{}", err);
            }
        },
    }
}
